define([
    'jquery',
    'ko',
    'underscore',
    'mage/storage',
    'mage/utils/template',
    'Magento_Checkout/js/model/resource-url-manager'
], function ($, ko, _, storage, mageTemplate, urlManager) {
    'use strict';

    return {
        defaults: {
            elementTmpl: 'Insign_PostCore/form/element/suggestions',
            suggestionTmpl: '${ $.text }',
            suggestionsDelay: 100,
            suggestionsCountry: "CH",
            valueKey: 'value',
            urlConfig: {},
            ignoreTmpls: {
                suggestionTmpl: true
            },
            minResults: 1,
            minValueLength: 2,
            imports: {
                selectedCountry: '${ $.provider }:shippingAddress.country_id'
            }
        },

        selectedCountry: null,
        suggestionsTimeout: null,
        suggestions: [],
        showSuggestionsDropdown: false,

        initialize: function() {
            this._super();

            this.value.subscribe(function(val) {
                if (this.selectedCountry() === this.suggestionsCountry && val.length >= this.minValueLength && this.focused()) {
                    clearTimeout(this.suggestionsTimeout);
                    this.suggestions([]);
                    this.suggestionsTimeout = setTimeout(this.getSuggestions.bind(this, val), this.suggestionsDelay);
                } else {
                    clearTimeout(this.suggestionsTimeout);
                    this.suggestions([]);
                }
            }, this);

            return this;
        },
        
        initObservable: function () {
            this._super();

            this.observe('suggestions selectedCountry showSuggestionsDropdown')

            return this;
        },

        focusSuggestionsInput: function() {
            this.showSuggestionsDropdown(true);
            if (this.selectedCountry() === this.suggestionsCountry && this.value().length >= this.minValueLength && this.suggestions().length == 0) { // load suggestions on focus, when empty
                clearTimeout(this.suggestionsTimeout);
                this.suggestionsTimeout = setTimeout(this.getSuggestions.bind(this, this.value()), this.suggestionsDelay);
            }
        },

        inputKeyDown: function(element, data, event) {
            if (event.keyCode == 9) { // tab key pressed
                this.showSuggestionsDropdown(false);
            } else if (event.keyCode == 40) {
                if (this.focused()) { // arrow down pressed
                    event.preventDefault();
                    $(element).find('.suggestions-list ul li:first-child a').focus();
                } else {
                    if (!$(document.activeElement.parentElement).is(':last-child')) {
                        event.preventDefault();
                        $(document.activeElement.parentElement.nextElementSibling).find('a').focus();
                    }
                }
            } else if (event.keyCode == 38) { // arrow up pressed
                if (!this.focused()) {
                    event.preventDefault();
                    if ($(document.activeElement.parentElement).is(':first-child')) {
                        this.focused(true);
                    } else {
                        $(document.activeElement.parentElement.previousElementSibling).find('a').focus();
                    }
                }
            }
            return true;
        },

        closeSuggestionsDropdown: function() {
            this.showSuggestionsDropdown(false);
        },

        suggestionsFormatter: function(suggestions, value) {
            var self = this;
            return suggestions.map(function(suggestion) {
                if (typeof suggestion === 'string') {
                    return {
                        text: suggestion,
                        html: suggestion.replace(new RegExp(value, 'ig'), function(match) {return "<strong>" + match + "</strong>"})
                    }
                } else {
                    suggestion.html = mageTemplate.template(self.suggestionTmpl, suggestion).replace(new RegExp(value, 'ig'), function(match) {return "<strong>" + match + "</strong>"});
                    return suggestion;
                }
            });
        },

        selectSuggestion: function(suggestion, event) {
            event.preventDefault();
            this.value(suggestion[this.valueKey]);
            this.focused(true);
        },

        getSuggestions: function(val) {
            var serviceUrl = urlManager.getUrl(this.urlConfig, {}),
                payload = JSON.stringify({
                    zipCity: val
                }),
                self = this;
                
            storage.post(
                serviceUrl,
                payload,
                false
            ).done(function (result) {
                if (val == self.value()) { // only add ned suggestions, if value is still the same, that was searched for
                    self.showSuggestionsDropdown(true);
                    self.suggestions(self.suggestionsFormatter(result/*.slice(0, 10)*/, val));
                }
            }).fail(function (response) {
                console.error(response);
            });
        }

    };
});
